unit Unit1;

interface

uses
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, CADtoDXF, CADImage, DXFConv, sgConsts;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    sgPaintBox: TImage;
    Panel2: TPanel;
    btnCreateDXFDirectly: TButton;
    SaveDialog: TSaveDialog;
    procedure btnCreateDXFDirectlyClick(Sender: TObject);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.btnCreateDXFDirectlyClick(Sender: TObject);
{ Creates new CAD image, adds entities with the specified
  linetype, color, layer etc. and save this file to DXF.   }
const
  cnstLayerName: string = 'new_Layer';
  cnstSpline: string = 'spline_Layer';
  cnstLTypeName: string = 'new_LType';
  cnstStyleChinese: string = 'SimSun_font_style';
  cnstKnots: array [1..12] of Single = (0,0,0,0, 9.36936792, 17.27242735, 26.43203326, 34.54490857, 43.56494422, 43.56494422, 43.56494422, 43.56494422);
  cnstControls: array [1..8] of TFPoint = (
    (X:193.7569940313442; Y:215.218107688496; Z:0.0),
    (X:190.008602213157; Y:216.4199761372127; Z:0.0),
    (X:183.0984428627155; Y:218.6356203125639; Z:0.0),
    (X:182.2793581329382; Y:203.2744864580255; Z:0.0),
    (X:195.7311751699836; Y:211.4104840390326; Z:0.0),
    (X:194.9308559557102; Y:195.9080522313814; Z:0.0),
    (X:188.1155501874722; Y:198.1118624085731; Z:0.0),
    (X:184.5274675101238; Y:199.2721116167301; Z:0.0));
var
  I: Integer;
  vCADFile: TsgCADImage;
  vArc: TsgDXFArc;
  vAttdef: TsgDXFAttdef;
  vAttrib: TsgDXFAttrib;
  vBlock: TsgDXFBlock;
  vCircle: TsgDXFCircle;
  vExp: TsgCADtoDXF;
  vHatch: TsgCADHatch;
  vInsert: TsgDXFInsert;
  vLayer: TsgDXFLayer;
  vLine: TsgDXFLine;
  vLType: TsgDXFLineType;
  vLWPolyline: TsgDXFLWPolyline;
  vMText: TsgDXFMText;
  vPolyline: TsgDXFPolyline;
  vSolid: TsgDXFSolid;
  vSpline: TsgDXFSpline;
  vStyle: TsgDXFStyle;
  vText: TsgDXFText;
  v2DBList: Tsg2DBoundaryList;
  v2DPolyline: Tsg2DPolyline;
  vPoint2D: TF2DPoint;
  vPPoint3D: PFPoint;
  vHPData: PsgHatchPatternData;

  procedure Add2DVertex(ALWPoly: TsgDXFLWPolyline; const X,Y, ABulge: Double);
  var
    vVertex: TsgDXFVertex;
  begin
    vVertex := TsgDXFVertex.Create;
    vVertex.Point := MakeFPoint(X, Y, 0);
    vVertex.Bulge := ABulge;
    ALWPoly.AddEntity(vVertex);
  end;

  procedure Add3DVertex(APoly: TsgDXFPolyline; const X,Y,Z: Double);
  var
    vVertex: TsgDXFVertex;
  begin
    vVertex := TsgDXFVertex.Create;
    vVertex.Point := MakeFPoint(X, Y, Z);
    APoly.AddEntity(vVertex);
  end;

  procedure AddFaceByVertices(APoly: TsgDXFPolyline; const N1,N2,N3,N4: Integer);
  var
    vVertex: TsgDXFVertex;
  begin
    vVertex := TsgDXFVertex.Create;
    vVertex.PolyFaceVertexIndex1  := N1;
    vVertex.PolyFaceVertexIndex2  := N2;
    vVertex.PolyFaceVertexIndex3  := N3;
    vVertex.PolyFaceVertexIndex4  := N4;
    vVertex.Flags := 128;
    vPolyline.AddEntity(vVertex);
  end;

  function AddEntityToLayout(AEntity: TsgDXFEntity; const ALayoutIndex: Integer): Boolean;
  begin
    Result := False;
    if ALayoutIndex < vCADFile.LayoutsCount then
    begin
      if Assigned(vCADFile.Converter.OnCreate) then
        vCADFile.Converter.OnCreate(AEntity);
      vCADFile.Converter.Loads(AEntity);
      vCADFile.Layouts[ALayoutIndex].AddEntity(AEntity);
      Result := True;
    end;
  end;

begin
  if not SaveDialog.Execute or (SaveDialog.FileName = '') then
    Exit;
  vCADFile := TsgCADImage.Create;
  try
    vCADFile.Converter.InitializeSections;

    { Adding the new layer with color to image-converter }
    vLayer := TsgDXFLayer.Create;
    vLayer.Name := cnstSpline;
    vLayer.Color := $7F9FFF;
    vCADFile.Converter.Sections[csLayers].AddEntity(vLayer);

    { Adding the new linetype to image-converter }
    vLType := TsgDXFLineType.Create;
    vLType.Name := cnstLTypeName;
    vLType.Lines.AddTick(25);       // length of the "solid" part of a line
    vLType.Lines.AddTick(-12.5);    // length of the "empty" part of a line
    vCADFile.Converter.Sections[csLTypes].AddEntity(vLType);

    { Adding the new text style to image-converter }
    vStyle := TsgDXFStyle.Create;
    vStyle.Name := cnstStyleChinese;
    vStyle.PrimaryFont := 'SimSun';// setting your Unicode-font here (file with big size from folder "$WINDOWS\Fonts")
    vCADFile.Converter.Sections[csStyles].AddEntity(vStyle);

    { Adding the TEXT entity }
    vText := TsgDXFText.Create;
    vText.Text  := 'Text';
    vText.Point := MakeFPoint(10, 280, 0);
    vText.Height := 10;
    vText.SetColor(clTeal);                 // color
    vText.Layer := vCADFile.Converter.LayerByName(cnstLayerName);// Layer will be created automatically
    if not AddEntityToLayout(vText, 0) then vText.Free;

    { Adding the TEXT entity with unicode & chinese text}
    vText := TsgDXFText.Create;
    vText.Text  := 'Unicode+Chinese: \U+00B11\U+00B0\U+7777';
    vText.Style := vCADFile.Converter.StyleByName(cnstStyleChinese);
    vText.Point := MakeFPoint(10, 265, 0);
    vText.Height := 10;
    vText.SetColor(clTeal);                 // color
    vText.Layer := vCADFile.Converter.LayerByName(cnstLayerName);// Layer will be created automatically
    if not AddEntityToLayout(vText, 0) then vText.Free;

    { Adding the MTEXT entity }
    vMText := TsgDXFMText.Create;
    vMText.Text  := 'MText (center)';
    vMText.Point := MakeFPoint(125, 250, 0);
    vMText.RectWidth := 250;
    vMText.Align := 5;
    vMText.Height := 15;
    vMText.SetColor(clTeal);                 // color
    vMText.Layer := vCADFile.Converter.LayerByName(cnstLayerName);// Layer will be created automatically
    if not AddEntityToLayout(vMText, 0) then vMText.Free;

    { Adding the LINE entity }
    vLine := TsgDXFLine.Create;
    vLine.Point  := MakeFPoint( 80, 100, 0);// first point
    vLine.Point1 := MakeFPoint(150, 150, 0);// second point
    vLine.SetColor(clBlue);                 // color
    vLine.Layer := vCADFile.Converter.LayerByName(cnstLayerName);// Layer will be created automatically
    vLine.SetLType(vCADFile.Converter.LTypeByName(cnstLTypeName));
    if not AddEntityToLayout(vLine, 0) then vLine.Free;

    { Adding the POLYLINE entity }
    vPolyline := TsgDXFPolyline.Create;
    vPolyline.Closed := True;
    vPolyline.LineWeight := 0.01*sgDXFLineWeights[High(sgDXFLineWeights)];
    Add3DVertex(vPolyline, 0, 0, 0);
    Add3DVertex(vPolyline, 0, 300, 0);
    Add3DVertex(vPolyline, 250, 300, 0);
    Add3DVertex(vPolyline, 250, 0, 0);
    vPolyline.SetColor(clMaroon);            // color
    vPolyline.Layer := vCADFile.Converter.LayerByName(cnstLayerName);
    if not AddEntityToLayout(vPolyline, 0) then vPolyline.Free;

    { Adding the Polyface mesh entity }
    vPolyline := TsgDXFPolyline.Create;
    vPolyline.Flags := 64;// the POLYLINE is a polyface mesh
    vPolyline.MeshM := 10;// number of vertices
    vPolyline.MeshN := 3; // number of feces
    // vertices
    Add3DVertex(vPolyline, 157, 177, 0);
    Add3DVertex(vPolyline, 190, 196, 0);
    Add3DVertex(vPolyline, 223, 176, 0);
    Add3DVertex(vPolyline, 209, 203, 0);
    Add3DVertex(vPolyline, 234, 218, 0);
    Add3DVertex(vPolyline, 201, 218, 0);
    Add3DVertex(vPolyline, 190, 244, 0);
    Add3DVertex(vPolyline, 179, 219, 0);
    Add3DVertex(vPolyline, 148, 219, 0);
    Add3DVertex(vPolyline, 172, 204, 0);
    // faces
    AddFaceByVertices(vPolyline, 1, 5, 8, 0);
    AddFaceByVertices(vPolyline, 3, 7, 10, 0);
    AddFaceByVertices(vPolyline, 9, 8, 10, 0);
    vPolyline.SetColor(clRed);   // color
    vPolyline.Layer := vCADFile.Converter.LayerByName(cnstLayerName);
    if not AddEntityToLayout(vPolyline, 0) then vPolyline.Free;

    { Adding the LWPOLYLINE entity with bulges and globalwidth}
    vLWPolyline := TsgDXFLWPolyline.Create;
    vLWPolyline.Closed := True;
    vLWPolyline.GlobalWidth := 4;
    Add2DVertex(vLWPolyline, -50, -50, 0.2);
    Add2DVertex(vLWPolyline, -50, 350, 0.2);
    Add2DVertex(vLWPolyline, 300, 350, 0.2);
    Add2DVertex(vLWPolyline, 300, -50, 0.2);
    vLWPolyline.SetColor(clNavy);            // color
    vLWPolyline.SetLType(vCADFile.Converter.LTypeByName(cnstLTypeName));
    vLWPolyline.Layer := vCADFile.Converter.LayerByName(cnstLayerName);
    if not AddEntityToLayout(vLWPolyline, 0) then vLWPolyline.Free;

    { Adding the SPLINE entity (spline by control points and knots) with default color}
    vSpline := TsgDXFSpline.Create;
    for I := Low(cnstControls) to High(cnstControls) do
    begin
      New(vPPoint3D);
      vPPoint3D^ := cnstControls[I];
      vSpline.Controls.Add(vPPoint3D);
    end;
    for I := Low(cnstKnots) to High(cnstKnots) do
      vSpline.Knots.Add(Pointer(cnstKnots[I]));
    vSpline.Layer := vCADFile.Converter.LayerByName(cnstSpline);
    if not AddEntityToLayout(vSpline, 0) then vSpline.Free;

    { Adding the ARC entity }
    vArc := TsgDXFArc.Create;
    vArc.StartAngle := 60;                  // start angle in degrees
    vArc.EndAngle := 210;                   // end angle in degrees
    vArc.Radius := 50;                      // radius
    vArc.Point := MakeFPoint(110, 160, 0);  // center point
    vArc.SetColor(clGreen);                 // color
    vArc.Layer := vCADFile.Converter.LayerByName(cnstLayerName);
    vArc.SetLType(vCADFile.Converter.LTypeByName(cnstLTypeName));
    if not AddEntityToLayout(vArc, 0) then vArc.Free;

    { Adding the CIRCLE entity }
    vCircle := TsgDXFCircle.Create;
    vCircle.LineTypeScale := 0.1;
    vCircle.Radius := 30;                      // radius
    vCircle.Point := MakeFPoint(110, 160, 0);  // center point
    vCircle.SetColor(clGreen);                 // color
    vCircle.Layer := vCADFile.Converter.LayerByName(cnstLayerName);
    vCircle.SetLType(vCADFile.Converter.LTypeByName(cnstLTypeName));
    if not AddEntityToLayout(vCircle, 0) then vCircle.Free;

    { Adding the SOLID entity }
    vSolid := TsgDXFSolid.Create;
    vSolid.Point := MakeFPoint(20, 50, 0);
    vSolid.Point1 := MakeFPoint(80, 50, 0);
    vSolid.Point2 := MakeFPoint(80, 90, 0);
    vSolid.Point3 := MakeFPoint(20, 90, 0);
    vSolid.SetColor(clNavy);                  // color
    vSolid.Layer := vCADFile.Converter.LayerByName(cnstLayerName);
    if not AddEntityToLayout(vSolid, 0) then vSolid.Free;

    { Adding the HATCH entity }
    vHatch := TsgCADHatch.Create;
    vHatch.HatchName := 'ANSI31';
    v2DBList := Tsg2DBoundaryList.Create;
    v2DBList.BoundaryType := 7;// Polyline type
    vHatch.BoundaryData.Add(v2DBList);
    v2DPolyline := Tsg2DPolyline.Create;
    v2DBList.Add(v2DPolyline);
    vPoint2D := MakeF2DPoint(200.0, 40.0);
    v2DPolyline.AddVertex(vPoint2D);
    vPoint2D := MakeF2DPoint(93.0,29.0);
    v2DPolyline.AddVertex(vPoint2D);
    vPoint2D := MakeF2DPoint(150.0, 120.0);
    v2DPolyline.AddVertex(vPoint2D);
    vPoint2D := MakeF2DPoint(200.0,40.0);
    v2DPolyline.AddVertex(vPoint2D);
    vHatch.SetColor(clBlue);
    New(vHPData);
    vHPData.BaseP := MakeFPoint(0, 0, 0);
    vHPData.Offset := MakeFPoint(-0.1, 0.1, 0);
    vHPData.LineAngle := 45.0;
    vHPData.IsDash := False;
    vHPData.Lines := nil;
    vHPData.DashNum := 0;
    vHatch.HatchPatternData.Add(vHPData);
    vHatch.Layer := vCADFile.Converter.LayerByName(cnstLayerName);
    if not AddEntityToLayout(vHatch, 0) then vHatch.Free;

    { Adding the INSERT with ATTRIBs and the BLOCK with ATTDEFs }
    vInsert := TsgDXFInsert.Create;
    vBlock := TsgDXFBlock.Create;
    vBlock.Name := 'AttribsTest';
    vBlock.Flags := 2;// bit coded flag 2 = block has non-constant attribute definitions
    vAttdef := TsgDXFAttdef.Create;
    vAttdef.Tag := 'ATTDEF#1';
    vAttdef.Text := 'This is ATTDEF text';
    vAttdef.Height := 1;
    if Assigned(vCADFile.Converter.OnCreate) then
      vCADFile.Converter.OnCreate(vAttdef);
    vCADFile.Converter.Loads(vAttdef);
    vBlock.AddEntity(vAttdef);
    if Assigned(vCADFile.Converter.OnCreate) then
      vCADFile.Converter.OnCreate(vBlock);
    vCADFile.Converter.Loads(vBlock);
    vCADFile.Converter.Sections[csBlocks].AddEntity(vBlock);
    vInsert.Block := vBlock;
    vInsert.Point := MakeFPoint(90, 90, 0);
    vAttrib := TsgDXFAttrib.Create;
    vAttrib.Point := MakeFPoint(10, 10, 0);
    vAttrib.Tag := 'ATTDEF#1';// = vAttdef.Tag
    vAttrib.Text := 'ATTRIB text!';
    vAttrib.Height := 20;
    if Assigned(vCADFile.Converter.OnCreate) then
      vCADFile.Converter.OnCreate(vAttrib);
    vCADFile.Converter.Loads(vAttrib);
    vInsert.Attribs.Add(vAttrib);
    if not AddEntityToLayout(vInsert, 0) then vInsert.Free;

    { Apply new drawing extents }
    vCADFile.GetExtents;

    { Copying new drawing to the sgPaintBox }
    sgPaintBox.Picture.Graphic := vCADFile;
    sgPaintBox.Width := Round(vCADFile.AbsWidth);
    sgPaintBox.Height := Round(vCADFile.AbsHeight);
    sgPaintBox.Invalidate;
    vExp := TsgCADtoDXF.Create(TsgCADImage(sgPaintBox.Picture.Graphic));
    vExp.SaveToFile(SaveDialog.FileName);
    vExp.Free;
  finally
    vCADFile.Free;
  end;
end;

end.
